"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encode = void 0;
const tslib_1 = require("tslib");
const operations = tslib_1.__importStar(require("../../operations"));
const clock_1 = require("../../clock");
const toBase64_1 = require("../../../util/base64/toBase64");
const timestamp = (sid, time, ts) => {
    const tsSessionId = ts.sid;
    if (tsSessionId === 1)
        return ts.time;
    else if (tsSessionId === sid && ts.time >= time)
        return time - ts.time - 1;
    return [tsSessionId, ts.time];
};
const timespan = (sid, time, span) => {
    const ts = timestamp(sid, time, span);
    if (ts instanceof Array) {
        ts.push(span.span);
        return ts;
    }
    return [ts, span.span];
};
const encode = (patch) => {
    const id = patch.getId();
    if (!id)
        throw new Error('PATCH_EMPTY');
    const sid = id.sid;
    const time = id.time;
    const header = sid === 1 ? [time] : [[sid, time]];
    const meta = patch.meta;
    if (meta !== undefined)
        header.push(meta);
    const res = [header];
    for (const op of patch.ops) {
        if (op instanceof operations.NewConOp) {
            const val = op.val;
            if (val instanceof clock_1.Timestamp) {
                res.push([0, timestamp(sid, time, val), true]);
            }
            else if (val === undefined) {
                res.push([0]);
            }
            else {
                res.push([0, val]);
            }
        }
        else if (op instanceof operations.NewValOp) {
            res.push([1, timestamp(sid, time, op.val)]);
        }
        else if (op instanceof operations.NewObjOp) {
            res.push([2]);
        }
        else if (op instanceof operations.NewVecOp) {
            res.push([3]);
        }
        else if (op instanceof operations.NewStrOp) {
            res.push([4]);
        }
        else if (op instanceof operations.NewBinOp) {
            res.push([5]);
        }
        else if (op instanceof operations.NewArrOp) {
            res.push([6]);
        }
        else if (op instanceof operations.InsValOp) {
            res.push([9, timestamp(sid, time, op.obj), timestamp(sid, time, op.val)]);
        }
        else if (op instanceof operations.InsObjOp) {
            const tuples = [];
            for (const [key, value] of op.data)
                tuples.push([key, timestamp(sid, time, value)]);
            const operation = [
                10,
                timestamp(sid, time, op.obj),
                tuples,
            ];
            res.push(operation);
        }
        else if (op instanceof operations.InsVecOp) {
            const tuples = [];
            for (const [key, value] of op.data)
                tuples.push([key, timestamp(sid, time, value)]);
            const operation = [
                11,
                timestamp(sid, time, op.obj),
                tuples,
            ];
            res.push(operation);
        }
        else if (op instanceof operations.InsStrOp) {
            const operation = [
                12,
                timestamp(sid, time, op.obj),
                timestamp(sid, time, op.ref),
                op.data,
            ];
            res.push(operation);
        }
        else if (op instanceof operations.InsBinOp) {
            const operation = [
                13,
                timestamp(sid, time, op.obj),
                timestamp(sid, time, op.ref),
                (0, toBase64_1.toBase64)(op.data),
            ];
            res.push(operation);
        }
        else if (op instanceof operations.InsArrOp) {
            const elements = [];
            for (const element of op.data)
                elements.push(timestamp(sid, time, element));
            const operation = [
                14,
                timestamp(sid, time, op.obj),
                timestamp(sid, time, op.ref),
                elements,
            ];
            res.push(operation);
        }
        else if (op instanceof operations.DelOp) {
            const operation = [
                16,
                timestamp(sid, time, op.obj),
                op.what.map((span) => timespan(sid, time, span)),
            ];
            res.push(operation);
        }
        else if (op instanceof operations.NopOp) {
            const operation = [17];
            const len = op.len;
            if (len > 1)
                operation.push(len);
            res.push(operation);
        }
    }
    return res;
};
exports.encode = encode;
